/*********************************************************************

popen  by tnl (tnl ta asia tod com)


Introduction:
    popen is a module for LiteStep that, like other popen implementations,
    lets you run a command-line command without showing a command prompt.

    The popen function itself was lifted from 
    http://home.mweb.co.za/sd/sdonovan/popen.zip


Installation/loading:
  classic:
    Put this somewhere in step.rc or its included files..

    LoadModule "x:\path to\the\popen.dll"

  NetLoadModule:

    *NetLoadModule popen-0.1


Usage:
    !popen <command>

  Example:
    !popen dir /b /on /ad "$ThemesDir$">"$ConfigDir$themeslist.txt"


Versions:
    0.1: initial version.


*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <io.h>
#include <windows.h>
#include <string.h>
#include "AggressiveOptimize.h"
#include "lsapi.h"
#include "popen.h"


/* RunSilent() is by Steven Szelei */
DWORD RunSilent(const char* strCommand)
{
	STARTUPINFO StartupInfo;
	PROCESS_INFORMATION ProcessInfo;
	char Args[4096];
	char *pEnvCMD = NULL;
	char *pDefaultCMD = "CMD.EXE";
	ULONG rc;

	memset(&StartupInfo, 0, sizeof(StartupInfo));
	StartupInfo.cb = sizeof(STARTUPINFO);
	StartupInfo.dwFlags = STARTF_USESHOWWINDOW;
	StartupInfo.wShowWindow = SW_HIDE;

	Args[0] = 0;

	pEnvCMD = getenv("COMSPEC");

	if(pEnvCMD){
		strcpy(Args, pEnvCMD);
	} else{
		strcpy(Args, pDefaultCMD);
	}

	/* "/c" option - Do the command then terminate the command window */
	strcat(Args, " /c ");
	/*the application you would like to run from the command window */
	strcat(Args, strCommand);

	if (!CreateProcess( NULL, Args, NULL, NULL, FALSE,
		CREATE_NEW_CONSOLE,
		NULL,
		NULL,
		&StartupInfo,
		&ProcessInfo))
	{
		return GetLastError();
	}

	WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
	if(!GetExitCodeProcess(ProcessInfo.hProcess, &rc))
		rc = 0;

	CloseHandle(ProcessInfo.hThread);
	CloseHandle(ProcessInfo.hProcess);

	return rc;
}

/* !popen <command> */
void bang_popen(HWND hWnd, LPCSTR args) {
    RunSilent(args);
}

__declspec(dllexport) int initModuleEx(HWND parent, HINSTANCE dllInst, LPCSTR szPath) {
    AddBangCommand("!popen", &bang_popen);
	return 0;
}

__declspec(dllexport) void quitModule(HINSTANCE dllInst) {
	RemoveBangCommand("!popen");
}

